﻿using System.Threading;
using System.Threading.Tasks;

namespace HeuristicLab.PluginInfrastructure {

  /// <summary>
  /// Interface for all runner implementations.
  /// </summary>
  public interface IRunnerHost {
    /// <summary>
    /// The runner state.
    /// </summary>
    RunnerState State { get; }

    /// <summary>
    /// Method to run the runner. This method is blocking.
    /// </summary>
    /// <param name="runner">A instance of an IRunner.</param>
    void Run(IRunner runner);

    /// <summary>
    /// Method to run the runner async. A CencellationToken can be specified.
    /// </summary>
    /// <param name="runner">A instance of an IRunner</param>
    /// <param name="token">The CancellationToken obtained from an CancellationTokenSource.</param>
    /// <returns>The task in which the process runs.</returns>
    Task RunAsync(IRunner runner, CancellationToken? token = null);

    /// <summary>
    /// Method to pause the runner and the isolated IApplication.
    /// </summary>
    void Pause();

    /// <summary>
    /// Method to resume a paused runner and the isolated IApplication.
    /// </summary>
    void Resume();
  }
}
