﻿using System.Diagnostics;
using HeuristicLab.PluginInfrastructure.Exceptions;

namespace HeuristicLab.PluginInfrastructure {
  /// <summary>
  /// IRunner for isolation with docker.
  /// </summary>
  public class DockerRunnerHost : RunnerHost {
    #region Constants
    private const string Docker = "docker";
    private const string ContainerStartup = "container run -i --rm ";
    private const string Mounting = @"--mount type=bind,source=/c/Users/,target=/Users ";
    private const string Image = "heuristiclab33:latest";
    private const string DockerExceptionMessage = "Docker is not running or image '" + Image + "' does not exists!";
    #endregion

    #region Constructors
    public DockerRunnerHost(bool doDockerAvailableCheck = true)
      : base(Docker, ContainerStartup + Mounting + Image, null, null, null) {
      if (doDockerAvailableCheck && !IsDockerAvailable())
        throw new DockerException(DockerExceptionMessage);
    }
    #endregion

    #region Helper
    private bool IsDockerAvailable() {
      var process = new Process {
        StartInfo = new ProcessStartInfo {
          FileName = Docker,
          Arguments = "image inspect " + Image,
          UseShellExecute = false,
          RedirectStandardOutput = true,
          RedirectStandardInput = false,
          RedirectStandardError = true,
          CreateNoWindow = false
        },
        EnableRaisingEvents = false
      };

      process.Start();
      process.BeginOutputReadLine();
      process.BeginErrorReadLine();
      process.WaitForExit();
      return process.ExitCode == 0;
    }
    #endregion
  }
}
